package config_test

import (
	. "github.com/pivotal-cf-experimental/mysql-metrics/config"

	"fmt"
	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"
	"io/ioutil"
	"os"
	"path/filepath"
	"strings"
)

var _ = Describe("MetricMappingConfig", func() {

	var (
		configFilepath string
		tempDir        string
		metricName     string
		key            string
		unit           string

		metricName2 string
		key2        string
		unit2       string
	)

	Describe("with a fully formed, correct yaml file", func() {
		BeforeEach(func() {
			metricName = "dummyMetric"
			key = "/p-mysql/performance/dummyMetric"
			unit = "jiff"

			metricName2 = "dummyMetric2"
			key2 = "/p-mysql/performance/dummyMetric2"
			unit2 = "jiff2"

			tempDir, err := ioutil.TempDir("", "")
			Expect(err).NotTo(HaveOccurred())

			configFilepath = filepath.Join(tempDir, "metric-mapping-config.yml")

			yaml := `
				---
				mappings:
				  %s:
				    key: %s
				    unit: %s
				  %s:
				    key: %s
				    unit: %s
			`
			yaml = strings.Replace(yaml, "\t", "", -1)
			configString := fmt.Sprintf(yaml, metricName, key, unit, metricName2, key2, unit2)

			err = ioutil.WriteFile(configFilepath, []byte(configString), os.ModePerm)
			Expect(err).NotTo(HaveOccurred())
		})

		AfterEach(func() {
			err := os.RemoveAll(tempDir)
			Expect(err).NotTo(HaveOccurred())
		})

		It("reads the config file", func() {
			var err error
			config := &MetricMappingConfig{}
			err = LoadFromFile(configFilepath, config)

			Expect(err).NotTo(HaveOccurred())
			Expect(config).NotTo(BeNil())
			Expect(config.Mappings).NotTo(BeNil())
			Expect(len(config.Mappings)).To(Equal(2))

			mapping := config.Mappings[metricName]
			Expect(mapping.Key).To(Equal(key))
			Expect(mapping.Unit).To(Equal(unit))
			mapping = config.Mappings[metricName2]
			Expect(mapping.Key).To(Equal(key2))
			Expect(mapping.Unit).To(Equal(unit2))
		})
	})

})
