package config

import (
	"database/sql"
	"io/ioutil"

	"fmt"
	_ "github.com/go-sql-driver/mysql"
	"github.com/pivotal-cf/mysql-diag/msg"
	yaml "gopkg.in/yaml.v2"
	"os"
)

type Config struct {
	Canary *CanaryConfig `yaml:"canary"`
	Mysql  MysqlConfig   `yaml:"mysql"`
}

type CanaryConfig struct {
	Username string `yaml:"username"`
	Password string `yaml:"password"`
	ApiPort  uint   `yaml:"api_port"`
}

type MysqlConfig struct {
	Username string       `yaml:"username"`
	Password string       `yaml:"password"`
	Port     uint         `yaml:"port"`
	Agent    *AgentConfig `yaml:"agent"`
	Nodes    []MysqlNode  `yaml:"nodes"`
}
type AgentConfig struct {
	Username string `yaml:"username"`
	Password string `yaml:"password"`
	Port     uint   `yaml:"port"`
}
type MysqlNode struct {
	Host string `yaml:"host"`
	Name string `yaml:"name"`
	UUID string `yaml:"uuid"`
}

func (mysqlConfig *MysqlConfig) Connection(node MysqlNode) *sql.DB {
	conn, err := sql.Open("mysql", fmt.Sprintf("%s:%s@tcp(%s:%d)/", mysqlConfig.Username, mysqlConfig.Password, node.Host, mysqlConfig.Port))
	if err != nil {
		msg.PrintfError("Database configuration problem: %v", err)
		os.Exit(1)
	}

	return conn
}

func (c *Config) HostsWithLogs() []string {
	result := []string{}
	for _, node := range c.Mysql.Nodes {
		result = append(result, node.Host)
	}

	return result
}

func LoadFromFile(filepath string) (*Config, error) {
	contents, err := ioutil.ReadFile(filepath)
	if err != nil {
		return nil, err
	}

	var c Config
	err = yaml.Unmarshal(contents, &c)
	if err != nil {
		return nil, err
	}

	return &c, nil
}
